'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var isStartOfSnippet = function isStartOfSnippet(line) {
  return line.trim().match(/```\W*(JavaScript|js|es6)/i);
};
var isEndOfSnippet = function isEndOfSnippet(line) {
  return line.trim() === '```';
};
var isSkip = function isSkip(line) {
  return line.trim() === '<!-- skip-example -->';
};
var isCodeSharedInFile = function isCodeSharedInFile(line) {
  return line.trim() === '<!-- share-code-between-examples -->';
};

function startNewSnippet(snippets, fileName, lineNumber) {
  var skip = snippets.skip;
  snippets.skip = false;

  return _extends(snippets, { snippets: snippets.snippets.concat([{ code: '', fileName: fileName, lineNumber: lineNumber, complete: false, skip: skip }]) });
}

function addLineToLastSnippet(line) {
  return function addLine(snippets) {
    var lastSnippet = snippets.snippets[snippets.snippets.length - 1];

    if (lastSnippet && !lastSnippet.complete) {
      lastSnippet.code += line + '\n';
    }

    return snippets;
  };
}

function endSnippet(snippets, fileName, lineNumber) {
  var lastSnippet = snippets.snippets[snippets.snippets.length - 1];

  if (lastSnippet) {
    lastSnippet.complete = true;
  }

  return snippets;
}

function skip(snippets) {
  snippets.skip = true;

  return snippets;
}

function shareCodeInFile(snippets) {
  snippets.shareCodeInFile = true;

  return snippets;
}

function parseLine(line) {
  if (isStartOfSnippet(line)) {
    return startNewSnippet;
  }

  if (isEndOfSnippet(line)) {
    return endSnippet;
  }

  if (isSkip(line)) {
    return skip;
  }

  if (isCodeSharedInFile(line)) {
    return shareCodeInFile;
  }

  return addLineToLastSnippet(line);
}

function parseCodeSnippets(args) {
  var contents = args.contents;
  var fileName = args.fileName;

  var initialState = {
    snippets: [],
    shareCodeInFile: false,
    complete: false
  };

  var results = contents.split('\n').map(parseLine).reduce(function (snippets, lineAction, index) {
    return lineAction(snippets, fileName, index + 1);
  }, initialState);

  var codeSnippets = results.snippets;

  var lastSnippet = codeSnippets[codeSnippets.length - 1];

  if (lastSnippet && !lastSnippet.complete) {
    throw new Error('Snippet parsing was incomplete');
  }

  return {
    fileName: fileName,
    codeSnippets: codeSnippets,
    shareCodeInFile: results.shareCodeInFile
  };
}

module.exports = parseCodeSnippets;